/**
 * gmain.c -- Tests the simulator with a gui
 * 
 * Copyright (C) 2011 Pantelis A. Frangoudis <pfrag@aueb.gr>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "gui.h"
#include <signal.h>

use_vc = 0;

void term_handler(int sig) {
	term_simulator();
	exit(0);
}

double run_round(int algo, int do_filter, int round) {

	theme t = {
		.ap_shape = SHAPE_TRI,
		.ap_color = COLOR_BLUE,
		.ap_size = 10,
		.cli_size = 6,
		.cli_shape = SHAPE_CROSS,
		.cli_color = COLOR_GREEN,
		.rad_color = COLOR_BLUE,
		.show_radius = 1,
		.show_channels = 1,
		.show_ap_id = 1
	};

	simsettings settings = {
		.terrainX = 1000,
		.terrainY = 1000,
		.num_aps = 200,
		.num_clients = 1000,
		.aps = NULL,
		.clients = NULL,
		.radius = 100,
		.theme = t
	};

	attack_scenario as = {
		.roaming_probability = 1.0,
		.truthful_probability = 0.2,
		.fake_group_size = 3,
		.colluding_group_size = 0,
		.managed_ap_probability = 0.8,
		.apply_trust = 0,
		.num_fake_channels = 3
	};

	init_simulator(
		settings.terrainX,
		settings.terrainY,
		settings.num_clients,
		settings.num_aps,
		settings.radius,
		1, //scenario 1: only clients report, but AP-AP edges are valid/considered
		&as);

	settings.aps = aps;
	settings.clients = clients;

	double dp = simulate(do_filter);

	fprintf(stderr, "max degree: %d\n", get_max_degree(reportedIG, next_fake_ap_id));
	fprintf(stderr, "%f\n", dp);

	
	//now run channel assignment algo
	assign_channels(reportedIG, reported_aps, next_fake_ap_id, algo);

	fprintf(stderr, "lmax: %f\tlsum: %f\n", get_lmax(IG, num_aps), get_lsum(IG, num_aps) );
/*
//	double cap = network_capacity(clients, num_clients);
//	double cap = get_lmax(IG, num_aps);
//	graph_stats(IG, num_aps);
	double cap = get_lsum(IG, num_aps);
*/
	char outfile[80];
	memset(outfile, 0, 80);
	sprintf(outfile, "clients-%d.txt", round);
	export_client_ca(clients, num_clients, outfile);

	memset(outfile, 0, 80);
	sprintf(outfile, "aps-%d.txt", round);
	export_ap_ca(aps, num_aps, outfile);

	term_simulator();

	//return cap;
	return 0;	
}

void run_with_gui(int argc, char **argv) {

	theme t = {
		.ap_shape = SHAPE_TRI,
		.ap_color = COLOR_BLUE,
		.ap_size = 10,
		.cli_size = 6,
		.cli_shape = SHAPE_CROSS,
		.cli_color = COLOR_GREEN,
		.rad_color = COLOR_BLUE,
		.show_radius = 1,
		.show_channels = 1,
		.show_ap_id = 1
	};

	simsettings settings = {
		.terrainX = 500,
		.terrainY = 500,
		.num_aps = 2,
		.num_clients = 14,
		.aps = NULL,
		.clients = NULL,
		.radius = 100,
		.theme = t
	};

	attack_scenario as = {
		.roaming_probability = 0.0,
		.truthful_probability = 1.0,
		.fake_group_size = 5,
		.colluding_group_size = 0,
		.managed_ap_probability = 1.0
	};
/*
	init_simulator(
		settings.terrainX,
		settings.terrainY,
		settings.num_clients,
		settings.num_aps,
		settings.radius,
		1,
		&as);
*/
a_fixed_topology();

	settings.aps = aps;
	settings.clients = clients;

	simulate(0);

	//now run channel assignment algo
	assign_channels(reportedIG, reported_aps, next_fake_ap_id, atoi(argv[1]));
	
	double cap = network_capacity(clients, num_clients);

	char outfile[80];
	memset(outfile, 0, 80);
	sprintf(outfile, "clients-fixed.txt");
	export_client_ca(clients, num_clients, outfile);

	memset(outfile, 0, 80);
	sprintf(outfile, "aps-fixed.txt");
	export_ap_ca(aps, num_aps, outfile);

	fprintf(stderr, "Simulator ended. Press Ctrl-C to quit\n");
	fprintf(stderr, "Network capacity: %f\n", cap);
	degree_distribution(IG, num_aps);
	show_gui(argc, argv, &settings);


	while(1) {}
}

int main(int argc, char **argv) {
	struct sigaction action;
	action.sa_handler = term_handler;
	sigemptyset (&action.sa_mask);
	action.sa_flags = 0;
	
	sigaction (SIGINT, &action, NULL);
	sigaction (SIGKILL, &action, NULL);
	sigaction (SIGTERM, &action, NULL);

	dsfmt_init_gen_rand(&RNG, 9999);

	int num_rounds = 1;
	int i = 0;
	double cap_sum = 0.0;


//  	for (i = 0; i < num_rounds; i++) {
//  		cap_sum += run_round(atoi(argv[1]), atoi(argv[2]), i);
//  	}
// // 
//  	fprintf(stderr, "Avg capacity: %f\n", cap_sum / num_rounds);

	run_with_gui(argc, argv);

	return 0;
}
