/**
 * gui.h -- Minimal GUI-based demonstration of the simulator terrain. Shows
 * where APs and clients are placed and cell range.
 * 
 * Copyright (C) 2011 Pantelis A. Frangoudis <pfrag@aueb.gr>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef _GUI_H_
#define _GUI_H_

#include <stdlib.h>
#include <gtk/gtk.h>
#include "simulator.h"

#define COLOR_RED 10
#define COLOR_BLUE 20
#define COLOR_GREEN 30
#define COLOR_BLACK 40
#define COLOR_WHITE 50
#define COLOR_YELLOW 60
#define COLOR_CYAN 70
#define COLOR_MAGENDA 80

#define SHAPE_RECT 10
#define SHAPE_CIRCLE 20
#define SHAPE_TRI 30
#define SHAPE_CROSS 40

typedef struct draw_theme_t {
	int ap_shape;
	int cli_shape;
	int ap_size;
	int cli_size;
	int ap_color;
	int managed_ap_color;
	int cli_color;
	int rad_color;
	int show_radius;
	int show_edges;
	int show_channels;
	int show_ap_id;
} theme;

typedef struct simulator_settings_t {
	double terrainX;
	double terrainY;
	int num_aps;
	int num_clients;
	struct access_point **aps;
	struct Client_ **clients;
	double radius;
	struct draw_theme_t theme;
} simsettings;

GdkPixmap *pixmap;

void get_color_from_theme(int color, GdkColor *gdkc);
void draw_topology(GtkWidget *widget, simsettings *s);
void draw_radius(theme *t, gdouble x, gdouble y, double radius);
void draw_client(theme *t, gdouble x, gdouble y);
void draw_ap(theme *t, gdouble x, gdouble y, int apid, int channel, int is_managed);
gboolean configure_event(GtkWidget *widget, GdkEventConfigure *event);
gboolean expose_event(GtkWidget *widget, GdkEventExpose *event, void *data);
int show_gui(int argc, char **argv, simsettings *s);

#endif
