/**
 * ig.h -- Coverage graph implementation. Each vertex is an AP and an edge 
 * represents overlapping coverage between two cells. The graph is implemented
 * as an adjacency list. For each edge, we keep track of those who reported it.
 * 
 * Copyright (C) 2011 Pantelis A. Frangoudis <pfrag@aueb.gr>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef _IG_H_
#define _IG_H_

#include "types.h"
#include "simulator.h"

Node *init_ig(AP **aps, int num_nodes);
Node *init_reported_ig(AP **aps, AP **fake_aps, int num_real_nodes, int num_fake_nodes);

void free_ig(Node *ig, int num_nodes);

int add_edge(Node *ig, int from, int to, double weight, int by_truthful);
void relax_edge(Node *ig, int from, int to, double weight);
void remove_edge(Node *ig, int from, int to);
double edge(Node *ig, int from, int to);
void add_reporter(Node *ig, int from, int to, Client *c);
#endif
