/**
 * main.c -- Simulator main file. Uncomment some code blocks to enable gui support
 * or showing results with confidence intervals. Using the gui is recommended only 
 * for demonstration/troubleshooting.
 * 
 * Copyright (C) 2011 Pantelis A. Frangoudis <pfrag@aueb.gr>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "simulator.h"
#include "simconfig.h"
#include "stats.h"
#include <errno.h>
#include "gui.h"

int main(int argc, char **argv) {

	struct simopts *so;

	int i = 0;
	int j = 0;

	if (argc != 2) {
		fprintf(stderr, "usage: ./wifisim conffile\n");
		exit(1);
	}

	so = load_configuration(argv[1]);

	/* print configuration */
/*
	fprintf(stderr, "Simulation settings: \n");
	fprintf(stderr, "---------------------\n");
	fprintf(stderr, "TerrainX: %f\nTerrainY: %f\nNum clients: %d\nNum APs: %d\nRange: %f\nRounds: %d\nBeta: %f\n", so->terrainX, so->terrainY, so->num_clients, so->num_aps, so->range, so->num_rounds, so->beta);
	fprintf(stderr, "Attacker ratio: %f\nDefection probability: %f\nManaged AP ratio: %f\nFake group size: %d\n", so->as.attacker_ratio, so->as.defection_probability, so->as.managed_ap_ratio, so->as.fake_group_size);
	fprintf(stderr, "---------------------\n\n");
*/

	struct round_stats **results = (struct round_stats **)malloc(so->iterations * sizeof(struct round_stats *));

	for (i = 0; i < so->iterations; i++) {
		/* 5/5/11: scenario must be '2': ap_edges, ap_reports, client_reports */
		//void init_simulator(double tX, double tY, int nc, int nap, double r, int scenario, attack_scenario *as, int sr, double b)
		init_simulator(so->terrainX, so->terrainY, so->num_clients, so->num_aps, so->range, 2, &so->as, so->num_rounds, so->beta);

		results[i] = simulate();

/////////////GUI///////////////////////
/*
	theme t = {
		.ap_shape = SHAPE_TRI,
		.ap_color = COLOR_BLUE,
		.managed_ap_color = COLOR_RED,
		.ap_size = 10,
		.cli_size = 6,
		.cli_shape = SHAPE_CROSS,
		.cli_color = COLOR_GREEN,
		.rad_color = COLOR_BLUE,
		.show_radius = 0,
		.show_edges = 1,		
		.show_channels = 0,
		.show_ap_id = 1
	};

	simsettings settings = {
		.terrainX = so->terrainX,
		.terrainY = so->terrainY,
		.num_aps = so->num_aps,
		.num_clients = so->num_clients,
		.aps = aps,
		.clients = clients,
		.radius = so->range,
		.theme = t
	};

	show_gui(argc, argv, &settings);
*/
///////////////////////////////////////

		term_simulator();
	}

	struct sim_stats *ss = get_stats(results, so->num_rounds, so->iterations);

	fprintf(stderr, "\n\n\tDP\t\tR(A)\t\tS(A)\t\tR(T)\t\tS(T)\t\tP(iso)\t\tR(all)\t\tS(all)\n");
	fprintf(stderr, "---------------------------------------------------------------------------------------------------------------------------\n");
	for (i = 0; i < so->num_rounds; i++) {
		fprintf(stderr, "%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n", i+1, ss[i].mean_dp, ss[i].mean_attacker_rep, ss[i].mean_attacker_score, ss[i].mean_truthful_rep, ss[i].mean_truthful_score, ss[i].mean_isolated, ss[i].mean_all_rep, ss[i].mean_all_score);
	}
	fprintf(stderr, "---------------------------------------------------------------------------------------------------------------------------\n");
	fprintf(stderr, "\tDP\t\tR(A)\t\tS(A)\t\tR(T)\t\tS(T)\t\tP(iso)\t\tR(all)\t\tS(all)\n");

	/* For AP-centric stats, run for 1 round */

/*
//	fprintf(stderr, "\n\n\tDP\t\tCI(DP)\t\tR(A)\t\tCI(RA)\t\tR(T)\t\tCI(RT)\n");
//	fprintf(stderr, "-----------------------------------------------------------\n");
	for (i = 0; i < so->num_rounds; i++) {
		fprintf(stderr, "%d\t%f\t%f\t%f\t%f\t%f\t%f\n", i + 1, ss[i].mean_dp, ss[i].ci_dp, ss[i].mean_attacker_rep, ss[i].ci_attacker_rep, ss[i].mean_truthful_rep, ss[i].ci_truthful_rep);
	}
//	fprintf(stderr, "-----------------------------------------------------------\n");
//	fprintf(stderr, "\tDP\t\tCI(DP)\t\tR(A)\t\tCI(RA)\t\tR(T)\t\tCI(RT)\n");
*/

	//free stuff
	for (i = 0; i < so->iterations; i++) {
		free(results[i]);
	}
	free(results);

	free(ss);

	free(so);
//	while(1){}
}

