/**
 * simconfig.h -- Simulator configuration. Includes a configuration file parser
 * which parses files with the following format: 
 * 
 * # comment
 * key1 value1
 * key2 value2
 * ....
 *
 * Keys are case-insensitive.
 *
 * Copyright (C) 2011 Pantelis A. Frangoudis <pfrag@aueb.gr>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef _SIMCONFIG_H_
#define _SIMCONFIG_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include "types.h"

#define DEF_TERRAIN_X 1000.0
#define DEF_TERRAIN_Y 1000.0
#define DEF_NUM_CLIENTS 1000
#define DEF_NUM_APS 100
#define DEF_BETA 0.8
#define DEF_NUM_ROUNDS 50
#define DEF_RANGE 100.0
#define DEF_ATTACKER_PROB 0.1
#define DEF_DEFECTION_PROB 0.3
#define DEF_MANAGED_AP_PROB 0.1
#define DEF_FAKE_GROUP_SIZE 5
#define DEF_ITERATIONS	5

struct simopts {
	double terrainX;
	double terrainY;
	int num_clients;
	int num_aps;
	double beta;
	int num_rounds;
	int iterations;
	double range;
	attack_scenario as;
};

int parse_line(char *line, char *key, char *value);
int parse_conffile(char *fname, char **confoptions, char **confvalues, int optlen);
struct simopts* load_configuration(char *fname);

#endif

