/**
 * stats.c -- Calculates simulation statistics (mean, confidence intervals).
 * 
 * Copyright (C) 2011 Pantelis A. Frangoudis <pfrag@aueb.gr>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "stats.h"

struct sim_stats *get_stats(struct round_stats **values, int numrounds, int numsamples) {
	int i = 0;
	int j = 0;

	double sum_dp = 0.0; //dp
	double sum_attacker_rep = 0.0; //attackers
	double sum_truthful_rep = 0.0; //truthful
	double sum_truthful_score = 0.0; //score
	double sum_attacker_score = 0.0;
	double sum_isolated = 0.0; //% isolated
	double sum_all_score = 0.0; //avg score for all types of users
	double sum_all_rep = 0.0; //avg rep for all types of users

	double sigma_dp = 0.0;
	double sigma_attacker_rep = 0.0;
	double sigma_truthful_score = 0.0;
	double sigma_truthful_rep = 0.0;
	double sigma_attacker_score = 0.0;
	double sigma_isolated = 0.0;
	double sigma_all_score = 0.0;
	double sigma_all_rep = 0.0;

	double mean_dp = 0.0;
	double mean_attacker_rep = 0.0;
	double mean_truthful_score = 0.0;
	double mean_truthful_rep = 0.0;
	double mean_attacker_score = 0.0;
	double mean_isolated = 0.0;
	double mean_all_score = 0.0;
	double mean_all_rep = 0.0;

	double sumdiff_dp = 0.0;
	double sumdiff_attacker_rep = 0.0;
	double sumdiff_truthful_score = 0.0;
	double sumdiff_truthful_rep = 0.0;
	double sumdiff_attacker_score = 0.0;
	double sumdiff_isolated = 0.0;
	double sumdiff_all_score = 0.0;
	double sumdiff_all_rep = 0.0;

	struct sim_stats *retval = (struct sim_stats*)malloc(numrounds * sizeof(struct sim_stats));

	double t = student_t_table_95[numsamples - 1];

	//for each round
	for (i = 0; i < numrounds; i++) {
		sum_dp = 0.0; //dp
		sum_attacker_rep = 0.0; //attackers
		sum_truthful_rep = 0.0; //truthful
		sum_truthful_score = 0.0; //score
		sum_attacker_score = 0.0;
		sum_isolated = 0.0; //% isolated
		sum_all_score = 0.0; //avg score for all types of users
		sum_all_rep = 0.0; //avg rep for all types of users
		
		sigma_dp = 0.0;
		sigma_attacker_rep = 0.0;
		sigma_truthful_score = 0.0;
		sigma_truthful_rep = 0.0;
		sigma_attacker_score = 0.0;
		sigma_isolated = 0.0;
		sigma_all_score = 0.0;
		sigma_all_rep = 0.0;

		mean_dp = 0.0;
		mean_attacker_rep = 0.0;
		mean_truthful_score = 0.0;
		mean_truthful_rep = 0.0;
		mean_attacker_score = 0.0;
		mean_isolated = 0.0;
		mean_all_score = 0.0;
		mean_all_rep = 0.0;

		sumdiff_dp = 0.0;
		sumdiff_attacker_rep = 0.0;
		sumdiff_truthful_score = 0.0;
		sumdiff_truthful_rep = 0.0;
		sumdiff_attacker_score = 0.0;
		sumdiff_isolated = 0.0;
		sumdiff_all_score = 0.0;
		sumdiff_all_rep = 0.0;

		//for each execution
		//calculate mean for each of the 4 columns
		for (j = 0; j < numsamples; j++) {
			sum_dp += values[j][i].dp;
			sum_attacker_rep += values[j][i].attacker_rep;
			sum_truthful_rep += values[j][i].truthful_rep;
			sum_attacker_score += values[j][i].attacker_score;
			sum_truthful_score += values[j][i].truthful_score;
			sum_isolated += values[j][i].isolated;
			sum_all_score += values[j][i].all_score;
			sum_all_rep += values[j][i].all_rep;
		}

		mean_dp = sum_dp/(double)numsamples;
		mean_attacker_rep = sum_attacker_rep/(double)numsamples;
		mean_truthful_rep = sum_truthful_rep/(double)numsamples;
		mean_truthful_score = sum_truthful_score/(double)numsamples;
		mean_attacker_score = sum_attacker_score/(double)numsamples;
		mean_isolated = sum_isolated/(double)numsamples;
		mean_all_score = sum_all_score/(double)numsamples;
		mean_all_rep = sum_all_rep/(double)numsamples;

		for (j = 0; j < numsamples; j++) {
			sumdiff_dp += (values[j][i].dp - mean_dp)*(values[j][i].dp - mean_dp);
			sumdiff_attacker_rep += (values[j][i].attacker_rep - mean_attacker_rep)*(values[j][i].attacker_rep - mean_attacker_rep);
			sumdiff_truthful_rep += (values[j][i].truthful_rep - mean_truthful_rep)*(values[j][i].truthful_rep - mean_truthful_rep);
			sumdiff_truthful_score += (values[j][i].truthful_score - mean_truthful_score)*(values[j][i].truthful_score - mean_truthful_score);
			sumdiff_attacker_score += (values[j][i].attacker_score - mean_attacker_score)*(values[j][i].attacker_score - mean_attacker_score);
			sumdiff_isolated += (values[j][i].isolated - mean_isolated)*(values[j][i].isolated - mean_isolated);
			sumdiff_all_rep += (values[j][i].all_rep - mean_all_rep)*(values[j][i].all_rep - mean_all_rep);
			sumdiff_all_score += (values[j][i].all_score - mean_all_score)*(values[j][i].all_score - mean_all_score);
		}

		sigma_dp = sqrt(sumdiff_dp/(double)numsamples);
		sigma_attacker_rep = sqrt(sumdiff_attacker_rep/(double)numsamples);
		sigma_truthful_rep = sqrt(sumdiff_truthful_rep/(double)numsamples);
		sigma_truthful_score = sqrt(sumdiff_truthful_score/(double)numsamples);
		sigma_attacker_score = sqrt(sumdiff_attacker_score/(double)numsamples);
		sigma_isolated = sqrt(sumdiff_isolated/(double)numsamples);
		sigma_all_rep = sqrt(sumdiff_all_rep/(double)numsamples);
		sigma_all_score = sqrt(sumdiff_all_score/(double)numsamples);

		retval[i].mean_dp = mean_dp;//mean detection prob.
		retval[i].ci_dp = 2.0*t*(sigma_dp/(double)numsamples);

		retval[i].mean_attacker_rep = mean_attacker_rep;
		retval[i].ci_attacker_rep = 2.0*t*(sigma_attacker_rep/(double)numsamples);

		retval[i].mean_truthful_rep = mean_truthful_rep;//mean reputation (truthful)
		retval[i].ci_truthful_rep = 2.0*t*(sigma_truthful_rep/(double)numsamples);

		retval[i].mean_truthful_score = mean_truthful_score;//mean score
		retval[i].ci_truthful_score = 2.0*t*(sigma_truthful_score/(double)numsamples);

		retval[i].mean_isolated = mean_isolated;
		retval[i].ci_isolated = 2.0*t*(sigma_isolated/(double)numsamples);

		retval[i].mean_attacker_score = mean_attacker_score;
		retval[i].ci_attacker_score = 2.0*t*(sigma_attacker_score/(double)numsamples);

		retval[i].mean_all_score = mean_all_score;
		retval[i].ci_all_score = 2.0*t*(sigma_all_score/(double)numsamples);

		retval[i].mean_all_rep = mean_all_rep;
		retval[i].ci_all_rep = 2.0*t*(sigma_all_rep/(double)numsamples);
	}

	return retval;
}

