/**
 * types.h -- Types/structs used by our simulator.
 * 
 * Copyright (C) 2011 Pantelis A. Frangoudis <pfrag@aueb.gr>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef _TYPES_H_
#define _TYPES_H_

#define AP_WEIGHT 1.1

typedef struct Client_ {
	int id;
	double x;
	double y;
	int scan_list_size;
	int fake_list_size;
	struct access_point **scan_list;
	struct access_point *assoc_ap;
	struct access_point **fake_list;

	double defection_probability;
	int will_attack;

	//used to calculate user's score per round [score = audited/reported]
	int reported_edges;
	int audited_edges;
	int reportedT1;
	int reportedT2;
	//client reputation
	double r;
} Client;


typedef struct access_point {
	int id;
	double x;
	double y;
	int num_assoc;
	int scan_list_size;
	int is_managed;
	struct Client_ **assoc_list;
	struct access_point **scan_list;

	int channel;
} AP;

typedef struct attack_scenario_ {
	int fake_group_size;
	int colluding_group_size;
	double trusted_ratio; /* not used. no one trusted. truthful guys: def probability = 0 */
	double managed_ap_ratio;
	double attacker_ratio;
	double defection_probability;
} attack_scenario;

typedef struct report_ {
	AP **scan_list;
	AP *assoc_ap;
} Report;

typedef struct ig_node_ {
	AP *ap;
	struct ig_edge_list_ *neighbors;
} Node;

typedef struct ig_edge_list_ {
	Node *to;
	double weight;
	int reported_by_truthful;
	Client **reporters;
	int num_reporters;
	struct ig_edge_list_ *next;
} Edge;

typedef struct terrain_block_ {
	int apnum;
	struct access_point **aps;
} Block;

struct round_stats {
	double dp;
	double truthful_rep;
	double attacker_rep;
	double isolated;
	double truthful_score;
	double attacker_score;
	double all_rep;
	double all_score;
};

struct sim_stats {
	double mean_dp;
	double ci_dp;

	double mean_attacker_rep;
	double ci_attacker_rep;

	double mean_truthful_rep;
	double ci_truthful_rep;

	double mean_truthful_score;
	double ci_truthful_score;

	double mean_isolated;
	double ci_isolated;

	double mean_attacker_score;
	double ci_attacker_score;

	double mean_all_score;
	double ci_all_score;

	double mean_all_rep;
	double ci_all_rep;
};

#endif
